@@ -80,9 +80,11 @@ struct _CheeseCameraPrivate
   GstElement *video_balance;
   GstElement *camera_tee, *effects_tee;
   GstElement *main_valve, *effects_valve;
+  gchar *current_effect_desc;
 
   gboolean is_recording;
   gboolean pipeline_is_playing;
+  gboolean effect_pipeline_is_playing;
   gchar *photo_filename;
 
   guint num_camera_devices;
@@ -182,7 +184,7 @@ cheese_camera_photo_data (CheeseCamera *camera, GstSample *sample)
  * @camera: the #CheeseCamera
  *
  * Process messages create by the @camera on the @bus. Emit
- * ::state-flags-changes if the state of the camera has changed.
+ * ::state-flags-changed if the state of the camera has changed.
  */
 static void
 cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *camera)
@@ -214,12 +216,15 @@ cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *ca
       gst_message_parse_error (message, &err, &debug);
 
       if (err && err->message) {
-        g_warning ("%s\n", err->message);
+        g_warning ("%s: %s\n", err->message, debug);
         g_error_free (err);
       } else {
         g_warning ("Unparsable GST_MESSAGE_ERROR message.\n");
       }
 
+      cheese_camera_stop (camera);
+      g_signal_emit (camera, camera_signals[STATE_FLAGS_CHANGED], 0,
+                     GST_STATE_NULL);
       g_free (debug);
       break;
     }
@@ -230,7 +235,11 @@ cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *ca
         GstState old, new;
         gst_message_parse_state_changed (message, &old, &new, NULL);
         if (new == GST_STATE_PLAYING)
+        {
           g_signal_emit (camera, camera_signals[STATE_FLAGS_CHANGED], 0, new);
+          cheese_camera_toggle_effects_pipeline (camera,
+                                            priv->effect_pipeline_is_playing);
+        }
       }
       break;
     }
@@ -528,6 +537,7 @@ cheese_camera_create_effects_preview_bin (CheeseCamera *camera, GError **error)
     cheese_camera_set_error_element_not_found (error, "effects_valve");
     return FALSE;
   }
+  g_object_set (G_OBJECT (priv->effects_valve), "drop", TRUE, NULL);
   if ((scale = gst_element_factory_make ("videoscale", "effects_scale")) == NULL)
   {
     cheese_camera_set_error_element_not_found (error, "videoscale");
@@ -599,6 +609,7 @@ cheese_camera_create_video_filter_bin (CheeseCamera *camera, GError **error)
     cheese_camera_set_error_element_not_found (error, "identity");
     return FALSE;
   }
+  priv->current_effect_desc = g_strdup("identity");
   if ((priv->video_balance = gst_element_factory_make ("videobalance", "video_balance")) == NULL)
   {
     cheese_camera_set_error_element_not_found (error, "videobalance");
@@ -782,6 +793,7 @@ cheese_camera_play (CheeseCamera *camera)
   CheeseCameraPrivate *priv   = CHEESE_CAMERA_GET_PRIVATE (camera);
   cheese_camera_set_new_caps (camera);
   g_object_set (priv->camera_source, "video-source", priv->video_source, NULL);
+  g_object_set (priv->main_valve, "drop", FALSE, NULL);
   gst_element_set_state (priv->camerabin, GST_STATE_PLAYING);
   priv->pipeline_is_playing = TRUE;
 }
@@ -915,16 +927,29 @@ cheese_camera_element_from_effect (CheeseCamera *camera, CheeseEffect *effect)
 void
 cheese_camera_set_effect (CheeseCamera *camera, CheeseEffect *effect)
 {
+  const gchar *effect_desc = cheese_effect_get_pipeline_desc (effect);
   GstElement *effect_filter;
 
   g_return_if_fail (CHEESE_IS_CAMERA (camera));
 
-  if (strcmp (cheese_effect_get_pipeline_desc (effect), "identity") == 0)
+  if (strcmp (camera->priv->current_effect_desc, effect_desc) == 0)
+  {
+    GST_INFO_OBJECT (camera, "Effect is: \"%s\", not updating", effect_desc);
+    return;
+  }
+
+  GST_INFO_OBJECT (camera, "Changing effect to: \"%s\"", effect_desc);
+
+  if (strcmp (effect_desc, "identity") == 0)
     effect_filter = gst_element_factory_make ("identity", "effect");
   else
     effect_filter = cheese_camera_element_from_effect (camera, effect);
   if (effect_filter != NULL)
+  {
     cheese_camera_change_effect_filter (camera, effect_filter);
+    g_free (camera->priv->current_effect_desc);
+    camera->priv->current_effect_desc = g_strdup(effect_desc);
+  }
 }
 
 /**
@@ -954,6 +979,7 @@ cheese_camera_toggle_effects_pipeline (CheeseCamera *camera, gboolean active)
     g_object_set (G_OBJECT (priv->effects_valve), "drop", TRUE, NULL);
     g_object_set (G_OBJECT (priv->main_valve), "drop", FALSE, NULL);
   }
+  priv->effect_pipeline_is_playing = active;
 }
 
 /**
@@ -1006,6 +1032,7 @@ cheese_camera_connect_effect_texture (CheeseCamera *camera, CheeseEffect *effect
   gst_element_set_state (effect_filter, GST_STATE_PLAYING);
   gst_element_set_state (display_queue, GST_STATE_PLAYING);
   gst_element_set_state (display_element, GST_STATE_PLAYING);
+  gst_element_set_locked_state (display_element, TRUE);
 
   if (!ok)
       g_warning ("Could not create effects pipeline");
@@ -1232,6 +1259,7 @@ cheese_camera_finalize (GObject *object)
 
   if (priv->photo_filename)
     g_free (priv->photo_filename);
+  g_free (priv->current_effect_desc);
   g_free (priv->device_node);
   g_boxed_free (CHEESE_TYPE_VIDEO_FORMAT, priv->current_format);
 
